@@ -13,7 +13,6 @@
 #define CONVERTER_H
 
 #include "Graph.h"
-#include "OutputParams.h"
 #include "support/trivstring.h"
 
 #include <vector>
@@ -29,10 +28,21 @@ class Buffer;
 class ErrorList;
 class Format;
 class Formats;
+class OutputParams;
+
+enum class Flavor : int;
+
+class ConversionException : public std::exception {
+public:
+	ConversionException() {}
+	virtual ~ConversionException() noexcept {}
+	const char * what() const noexcept override
+		{ return "Exception caught in conversion routine!"; }
+};
+
 
 typedef std::vector<Format const *> FormatList;
 
-
 ///
 class Converter {
 public:
@@ -48,7 +58,7 @@ public:
 	///
 	std::string const command() const { return command_; }
 	///
-	void setCommand(std::string const & command) { command_ = command; }
+	void setCommand(std::string const & command);
 	///
 	std::string const flags() const { return flags_; }
 	///
@@ -66,7 +76,7 @@ public:
 	///
 	std::string const latex_flavor() const { return latex_flavor_; }
 	///
-	bool xml() const { return xml_; }
+	bool docbook() const { return docbook_; }
 	///
 	bool need_aux() const { return need_aux_; }
 	/// Return whether or not the needauth option is set for this converter
@@ -100,8 +110,8 @@ private:
 	bool latex_;
 	/// The latex derivate
 	trivstring latex_flavor_;
-	/// The converter is xml
-	bool xml_;
+	/// The converter is DocBook
+	bool docbook_;
 	/// This converter needs the .aux files
 	bool need_aux_;
 	/// we need a "nice" file from the backend, c.f. OutputParams.nice.
@@ -128,6 +138,12 @@ public:
 	typedef std::vector<Converter> ConverterList;
 	///
 	typedef ConverterList::const_iterator const_iterator;
+	/// Return values for converter runs
+	enum RetVal {
+		SUCCESS = 0,
+		FAILURE = 1,
+		KILLED  = 1000
+	};
 
 	///
 	Converter const & get(int i) const { return converterlist_[i]; }
@@ -161,10 +177,10 @@ public:
 	///
 	Graph::EdgePath getPath(std::string const & from, std::string const & to);
 	///
-	OutputParams::FLAVOR getFlavor(Graph::EdgePath const & path,
-				       Buffer const * buffer = 0);
+	Flavor getFlavor(Graph::EdgePath const & path,
+					   Buffer const * buffer = nullptr) const;
 	///
-	std::string getHyperrefDriver(Graph::EdgePath const & path);
+	std::string getHyperrefDriver(Graph::EdgePath const & path) const;
 	/// Flags for converting files
 	enum ConversionFlags {
 		/// No special flags
@@ -175,17 +191,17 @@ public:
 		try_cache = 1 << 1
 	};
 	///
-	bool convert(Buffer const * buffer,
+	RetVal convert(Buffer const * buffer,
 		     support::FileName const & from_file, support::FileName const & to_file,
 		     support::FileName const & orig_from,
 		     std::string const & from_format, std::string const & to_format,
-		     ErrorList & errorList, int conversionflags = none);
+		     ErrorList & errorList, int conversionflags = none, bool includeall = false);
 	///
 	void update(Formats const & formats);
 	///
 	void updateLast(Formats const & formats);
 	///
-	bool formatIsUsed(std::string const & format);
+	bool formatIsUsed(std::string const & format) const;
 	///
 	const_iterator begin() const { return converterlist_.begin(); }
 	///
@@ -216,7 +232,7 @@ private:
 	bool scanLog(Buffer const & buffer, std::string const & command,
 		     support::FileName const & filename, ErrorList & errorList);
 	///
-	bool runLaTeX(Buffer const & buffer, std::string const & command,
+	RetVal runLaTeX(Buffer const & buffer, std::string const & command,
 		      OutputParams const &, ErrorList & errorList);
 	///
 	ConverterList converterlist_;
