@@ -223,8 +223,14 @@ void GuiWorkArea::init()
 	// Enables input methods for asian languages.
 	// Must be set when creating custom text editing widgets.
 	setAttribute(Qt::WA_InputMethodEnabled, true);
-	// obtain transformation information to reset it when LyX gets refocus
-	d->im_item_trans_ = d->im_->inputItemTransform();
+
+	// Initialize d->im_cursor_rect_
+	Point point;
+	Dimension dim;
+	d->buffer_view_->caretPosAndDim(point, dim);
+	int cur_x = point.x_ - dim.width();
+	int cur_y = point.y_ + dim.height();
+	d->im_cursor_rect_ = QRectF(cur_x, (cur_y - dim.height()) , 1, dim.height() );
 }
 
 
@@ -283,6 +289,9 @@ void GuiWorkArea::close()
 void GuiWorkArea::setFullScreen(bool full_screen)
 {
 	d->buffer_view_->setFullScreen(full_screen);
+
+	queryInputItemTransform();
+
 	if (full_screen && lyxrc.full_screen_scrollbar)
 		setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 	else
@@ -687,12 +696,22 @@ void GuiWorkArea::contextMenuEvent(QContextMenuEvent * e)
 
 void GuiWorkArea::focusInEvent(QFocusEvent * e)
 {
-	LYXERR(Debug::DEBUG, "GuiWorkArea::focusInEvent(): " << this << endl);
+	LYXERR(Debug::DEBUG, "GuiWorkArea::focusInEvent(): " << this << " reason() = " << e->reason() << endl);
 	if (d->lyx_view_->currentWorkArea() != this) {
 		d->lyx_view_->setCurrentWorkArea(this);
 		d->lyx_view_->currentWorkArea()->bufferView().buffer().updateBuffer();
 	}
 
+	// needs to reset IM item coordinates when focus in from dialogs or other apps
+	if ((e->reason() == Qt::PopupFocusReason || e->reason() == Qt::ActiveWindowFocusReason) &&
+		!(this->inDialogMode())) {
+		// Switched from most of dialogs or other apps, and not on a dialog (e.g. findreplaceadv)
+		d->item_trans_needs_reset_ = true;
+	} else {
+		// Switched from advanced search dialog or else (e.g. mouse event)
+		d->item_trans_needs_reset_ = false;
+	}
+
 	startBlinkingCaret();
 	QAbstractScrollArea::focusInEvent(e);
 }
@@ -971,7 +990,7 @@ void GuiWorkArea::generateSyntheticMouseEvent()
 	// Find the row at which we set the cursor.
 	RowList::const_iterator rit = pm.rows().begin();
 	RowList::const_iterator rlast = pm.rows().end();
-	int yy = pm.position() - pm.ascent();
+	int yy = pm.top();
 	for (--rlast; rit != rlast; ++rit) {
 		int h = rit->height();
 		if ((up && yy + h > 0)
@@ -1122,28 +1141,85 @@ void GuiWorkArea::resizeEvent(QResizeEvent * ev)
 }
 
 
+void GuiWorkArea::queryInputItemTransform()
+{
+	LYXERR(
+		   Debug::DEBUG,
+		   "item_trans_ is aquired: dx() = " << d->item_trans_.dx() <<
+		   " -> " << d->im_->inputItemTransform().dx() <<
+		   ", dy() = " << d->item_trans_.dy() <<
+		   " -> " << d->im_->inputItemTransform().dy()
+		   );
+
+	d->item_trans_ = d->im_->inputItemTransform();
+}
+
+
+void GuiWorkArea::Private::resetInputItemTransform()
+{
+	if (item_trans_needs_reset_) {
+		LYXERR(
+			   Debug::DEBUG,
+			   "(" << this <<
+			   ") item_trans_ is reset: dx() = " << im_->inputItemTransform().dx() <<
+			   " -> " << item_trans_.dx() <<
+			   ", dy() = " << im_->inputItemTransform().dy() <<
+			   " -> " << item_trans_.dy()
+			   );
+		im_->setInputItemTransform(item_trans_);
+		item_trans_needs_reset_ = false;
+	}
+}
+
+
+//#define DEBUG_PREEDIT
+
 void GuiWorkArea::Private::paintPreeditText(GuiPainter & pain)
 {
-	if (preedit_string_.empty())
+#ifdef DEBUG_PREEDIT
+	// check the language that current input method uses
+	QLocale::Language lang = im_->locale().language();
+	if (lang != im_lang_) {
+		LYXERR0("QLocale = " << QLocale::languageToString(lang));
+		im_lang_ = lang;
+	}
+#endif
+		
+	// Chinese IM may want cursor position even when preedit string is empty
+	// such a case is handled below
+	if (preedit_string_.empty() && im_->locale().language() != QLocale::Chinese)
 		return;
 
-	// FIXME: shall we use real_current_font here? (see #10478)
-	FontInfo const font = buffer_view_->cursor().getFont().fontInfo();
-	FontMetrics const & fm = theFontMetrics(font);
+	// lower margin of the preedit area to separate the candidate window
+	// report to IM the height of preedit rectangle larger than the actual by
+	// preedit_lower_margin so that the conversion suggestion window does not
+	// hide the underline of the preedit text
+	int preedit_lower_margin = 1;
+
 	Point point;
 	Dimension dim;
 	buffer_view_->caretPosAndDim(point, dim);
 	int cur_x = point.x_ - dim.width();
 	int cur_y = point.y_ + dim.height();
-	// lower margin of the preedit area to separate the candidate window
-	// report to IM the height of preedit rectangle larger than the actual by
-	// preedit_lower_margin so that the conversion suggestion window does not
-	// hide the underline of the preedit text
-	int preedit_lower_margin = 3;
-	// reset item transformation since it gets wrong after the item get
-	// lost and regain focus.
-	im_->setInputItemTransform(im_item_trans_);
+
+	if (preedit_string_.empty()) {
+		// Chinese input methods may exit here just obtaining im_cursor_rect
+		im_cursor_rect_ =
+			QRectF(cur_x, cur_y - dim.height(), 1, dim.height() + preedit_lower_margin);
+		im_->update(Qt::ImCursorRectangle);
+		return;
+	}
+
+	// reset item transformation since it can go wrong after the item gets
+	// lost and regains focus or after a new tab (dis)appears etc.
+	resetInputItemTransform();
+
+	// FIXME: shall we use real_current_font here? (see #10478)
+	FontInfo const font = buffer_view_->cursor().getFont().fontInfo();
+	FontMetrics const & fm = theFontMetrics(font);
+
 	// force fulldraw to remove previous paint remaining on screen
+    // FIXME: This is costly to do
 	buffer_view_->processUpdateFlags(Update::ForceDraw);
 
 	// get attributes of input method cursor.
@@ -1785,7 +1861,7 @@ bool TabWorkArea::setCurrentWorkArea(GuiWorkArea * work_area)
 	else
 		// Switch to the work area.
 		setCurrentIndex(index);
-	work_area->setFocus();
+	work_area->setFocus(Qt::OtherFocusReason);
 
 	return true;
 }
@@ -1811,6 +1887,13 @@ GuiWorkArea * TabWorkArea::addWorkArea(Buffer & buffer, GuiView & view)
 
 	updateTabTexts();
 
+	// obtain new input item coordinates in the new and old work areas
+	wa->queryInputItemTransform();
+	if (currentWorkArea())
+		currentWorkArea()->queryInputItemTransform();
+
+	view.setBusy(false);
+
 	return wa;
 }
 
@@ -1835,6 +1918,7 @@ bool TabWorkArea::removeWorkArea(GuiWorkArea * work_area)
 		else
 			// Show tabbar only if there's more than one tab.
 			showBar(count() > 1);
+		currentWorkArea()->queryInputItemTransform();
 	} else
 		lastWorkAreaRemoved();
 
@@ -1875,6 +1959,75 @@ void TabWorkArea::closeCurrentBuffer()
 }
 
 
+bool TabWorkArea::closeTabsToRight()
+{
+	if (clicked_tab_ == -1)
+		return false;
+
+	int const initialCurrentIndex = currentIndex();
+
+	while (count() - 1 > clicked_tab_) {
+		GuiWorkArea * wa = workArea(count() - 1);
+		LASSERT(wa, return false);
+		if (!wa->view().closeWorkArea(wa)) {
+			// closing cancelled, if possible, reset initial current tab index
+			if (initialCurrentIndex < count())
+				setCurrentIndex(initialCurrentIndex);
+			else
+				setCurrentIndex(clicked_tab_);
+			return false;
+		}
+	}
+	return true;
+}
+
+
+bool TabWorkArea::openEnclosingFolder()
+{
+
+	if (clicked_tab_ == -1)
+		return false;
+
+	Buffer const & buf = workArea(clicked_tab_)->bufferView().buffer();
+	showDirectory(buf.fileName().onlyPath());
+	return true;
+}
+
+
+bool TabWorkArea::closeTabsToLeft()
+{
+	if (clicked_tab_ == -1)
+		return false;
+
+	int const initialCurrentIndex = currentIndex();
+
+	int n = clicked_tab_;
+
+	for (int i = 0; i < n; ++i) {
+		GuiWorkArea * wa = workArea(0);
+		LASSERT(wa, return false);
+		if (!wa->view().closeWorkArea(wa)) {
+			// closing cancelled, if possible, reset initial current tab index
+			if (initialCurrentIndex - i >= 0)
+				setCurrentIndex(initialCurrentIndex - i);
+			else
+				setCurrentIndex(clicked_tab_ - i);
+			return false;
+		}
+	}
+	return true;
+}
+
+
+void TabWorkArea::closeOtherTabs()
+{
+	if (clicked_tab_ == -1)
+		return;
+
+	closeTabsToRight() && closeTabsToLeft();
+}
+
+
 void TabWorkArea::hideCurrentTab()
 {
 	GuiWorkArea * wa;
@@ -1902,6 +2055,22 @@ void TabWorkArea::closeTab(int index)
 }
 
 
+void TabWorkArea::moveToStartCurrentTab()
+{
+	if (clicked_tab_ == -1)
+		return;
+	tabBar()->moveTab(clicked_tab_, 0);
+}
+
+
+void TabWorkArea::moveToEndCurrentTab()
+{
+	if (clicked_tab_ == -1)
+		return;
+	tabBar()->moveTab(clicked_tab_, count() - 1);
+}
+
+
 ///
 class DisplayPath {
 public:
@@ -2149,19 +2318,42 @@ void TabWorkArea::showContextMenu(const QPoint & pos)
 
 	// show tab popup
 	QMenu popup;
-	popup.addAction(QIcon(getPixmap("images/", "hidetab", "svgz,png")),
-		qt_("&Hide Tab"), this, SLOT(hideCurrentTab()));
+	popup.addAction(qt_("&Hide Tab"), this, SLOT(hideCurrentTab()));
 
 	// we want to show the 'close' option only if this is not a child buffer.
 	Buffer const & buf = wa->bufferView().buffer();
 	if (!buf.parent())
-		popup.addAction(QIcon(getPixmap("images/", "closetab", "svgz,png")),
-			qt_("&Close Tab"), this, SLOT(closeCurrentBuffer()));
+		popup.addAction(qt_("&Close Tab"), this, SLOT(closeCurrentBuffer()));
+
+	popup.addSeparator();
+
+	QAction * closeOther = popup.addAction(qt_("Close &Other Tabs"), this, SLOT(closeOtherTabs()));
+	closeOther->setEnabled(clicked_tab_ != 0 || hasTabsToRight(clicked_tab_));
+	QAction * closeRight = popup.addAction(qt_("Close Tabs to the &Right"), this, SLOT(closeTabsToRight()));
+	closeRight->setEnabled(hasTabsToRight(clicked_tab_));
+	QAction * closeLeft = popup.addAction(qt_("Close Tabs to the &Left"), this, SLOT(closeTabsToLeft()));
+	closeLeft->setEnabled(clicked_tab_ != 0);
+
+	popup.addSeparator();
+
+	QAction * moveStart = popup.addAction(qt_("Move Tab to &Start"), this, SLOT(moveToStartCurrentTab()));
+	moveStart->setEnabled(closeLeft->isEnabled());
+	QAction * moveEnd = popup.addAction(qt_("Move Tab to &End"), this, SLOT(moveToEndCurrentTab()));
+	moveEnd->setEnabled(closeRight->isEnabled());
+
+	popup.addSeparator();
+
+	popup.addAction(qt_("Open Enclosing &Folder"), this, SLOT(openEnclosingFolder()));
+
 	popup.exec(tabBar()->mapToGlobal(pos));
 
 	clicked_tab_ = -1;
 }
 
+bool TabWorkArea::hasTabsToRight(int index) {
+	return count() - 1 > index;
+}
+
 
 void TabWorkArea::moveTab(int fromIndex, int toIndex)
 {
