@@ -5,21 +5,19 @@
  *
  * \author Angus Leeming
  * \author Georg Baum
- * \author Richard Heck
+ * \author Richard Kimberly Heck
  *
  * Full author contact details are available in file CREDITS.
  */
 
 #include <config.h>
-#include <algorithm>
-#include <functional>
-
 
 #include "InsetCommandParams.h"
 
 #include "InsetBibitem.h"
 #include "InsetBibtex.h"
 #include "InsetCitation.h"
+#include "InsetCounter.h"
 #include "InsetFloatList.h"
 #include "InsetHyperlink.h"
 #include "InsetInclude.h"
@@ -33,7 +31,6 @@
 #include "Buffer.h"
 #include "Encoding.h"
 #include "Lexer.h"
-#include "OutputParams.h"
 
 #include "frontends/alert.h"
 
@@ -45,6 +42,9 @@
 
 #include "support/lassert.h"
 
+#include <algorithm>
+#include <functional>
+
 using namespace std;
 using namespace lyx::support;
 
@@ -63,6 +63,8 @@ static ParamInfo const & findInfo(InsetCode code, string const & cmdName)
 		return InsetBibtex::findInfo(cmdName);
 	case CITE_CODE:
 		return InsetCitation::findInfo(cmdName);
+	case COUNTER_CODE:
+		return InsetCounter::findInfo(cmdName);
 	case FLOAT_LIST_CODE:
 		return InsetFloatList::findInfo(cmdName);
 	case HYPERLINK_CODE:
@@ -132,10 +134,10 @@ bool ParamInfo::hasParam(std::string const & name) const
 
 
 void ParamInfo::add(std::string const & name, ParamType type,
-                    ParamHandling handling, bool ignore,
+                    ParamHandling handling, bool ignoreval,
                     docstring default_value)
 {
-	info_.push_back(ParamData(name, type, handling, ignore, default_value));
+	info_.push_back(ParamData(name, type, handling, ignoreval, default_value));
 }
 
 
@@ -201,6 +203,8 @@ string InsetCommandParams::getDefaultCmd(InsetCode code)
 			return InsetBibtex::defaultCommand();
 		case CITE_CODE:
 			return InsetCitation::defaultCommand();
+		case COUNTER_CODE:
+			return InsetCounter::defaultCommand();
 		case FLOAT_LIST_CODE:
 			return InsetFloatList::defaultCommand();
 		case HYPERLINK_CODE:
@@ -238,6 +242,8 @@ bool InsetCommandParams::isCompatibleCommand(InsetCode code, string const & s)
 			return InsetBibtex::isCompatibleCommand(s);
 		case CITE_CODE:
 			return InsetCitation::isCompatibleCommand(s);
+		case COUNTER_CODE:
+			return InsetCounter::isCompatibleCommand(s);
 		case FLOAT_LIST_CODE:
 			return InsetFloatList::isCompatibleCommand(s);
 		case HYPERLINK_CODE:
@@ -282,7 +288,7 @@ void InsetCommandParams::setCmdName(string const & name)
 
 void InsetCommandParams::read(Lexer & lex)
 {
-	Read(lex, 0);
+	Read(lex, nullptr);
 }
 
 
@@ -316,7 +322,7 @@ void InsetCommandParams::Read(Lexer & lex, Buffer const * buffer)
 			preview_ = lex.getBool();
 			continue;
 		}
-		if (info_.hasParam(token)) {
+		if (hasParam(token)) {
 			lex.next(true);
 			docstring data = lex.getDocString();
 			if (buffer && token == "filename") {
@@ -356,7 +362,7 @@ void InsetCommandParams::Read(Lexer & lex, Buffer const * buffer)
 
 void InsetCommandParams::write(ostream & os) const
 {
-	Write(os, 0);
+	Write(os, nullptr);
 }
 
 
@@ -449,7 +455,21 @@ docstring InsetCommandParams::prepareCommand(OutputParams const & runparams,
 	// LATEXIFY, ESCAPE and NONE are mutually exclusive
 	if (handling & ParamInfo::HANDLING_LATEXIFY) {
 		// First handle backslash
-		result = subst(command, from_ascii("\\"), from_ascii("\\textbackslash{}"));
+		// we cannot replace yet with \textbackslash{}
+		// as the braces would be erroneously escaped
+		// in the following routines ("\textbackslash\{\}").
+		// So create a unique placeholder which is replaced
+		// in the end.
+		docstring bs = from_ascii("@LyXBackslash@");
+		// We are super-careful and assure the placeholder
+		// does not exist in the string
+		for (int i = 0; ; ++i) {
+			if (!contains(command, bs)) {
+				result = subst(command, from_ascii("\\"), bs);
+				break;
+			}
+			bs = from_ascii("@LyXBackslash") + i + '@';
+		}
 		// Then get LaTeX macros
 		pair<docstring, docstring> command_latexed =
 			runparams.encoding->latexString(result, runparams.dryrun);
@@ -487,6 +507,8 @@ docstring InsetCommandParams::prepareCommand(OutputParams const & runparams,
 							result.replace(pos, 1, backslash + chars_escape[k] + term);
 				}
 		}
+		// set in real backslash now
+		result = subst(result, bs, from_ascii("\\textbackslash{}"));
 	}
 	else if (handling & ParamInfo::HANDLING_ESCAPE)
 		result = escape(command);
@@ -494,8 +516,7 @@ docstring InsetCommandParams::prepareCommand(OutputParams const & runparams,
 		// we can only output characters covered by the current
 		// encoding!
 		docstring uncodable;
-		for (size_type i = 0 ; i < command.size() ; ++i) {
-			char_type c = command[i];
+		for (char_type c : command) {
 			try {
 				if (runparams.encoding->encodable(c))
 					result += c;
@@ -548,28 +569,33 @@ docstring InsetCommandParams::prepareCommand(OutputParams const & runparams,
 }
 
 
-docstring InsetCommandParams::getCommand(OutputParams const & runparams) const
+docstring InsetCommandParams::getCommand(OutputParams const & runparams, bool starred, bool unhandled) const
 {
 	docstring s = '\\' + from_ascii(cmdName_);
+	if (starred)
+		s += from_utf8("*");
 	bool noparam = true;
 	ParamInfo::const_iterator it  = info_.begin();
 	ParamInfo::const_iterator end = info_.end();
 	for (; it != end; ++it) {
 		std::string const & name = it->name();
+		ParamInfo::ParamHandling handling = unhandled ?
+					ParamInfo::HANDLING_NONE
+				      : it->handling();
 		switch (it->type()) {
 		case ParamInfo::LYX_INTERNAL:
 			break;
 
 		case ParamInfo::LATEX_REQUIRED: {
 			docstring const data =
-				prepareCommand(runparams, (*this)[name], it->handling());
+				prepareCommand(runparams, (*this)[name], handling);
 			s += '{' + data + '}';
 			noparam = false;
 			break;
 		}
 		case ParamInfo::LATEX_OPTIONAL: {
 			docstring data =
-				prepareCommand(runparams, (*this)[name], it->handling());
+				prepareCommand(runparams, (*this)[name], handling);
 			if (!data.empty()) {
 				s += '[' + protectArgument(data) + ']';
 				noparam = false;
@@ -599,10 +625,24 @@ docstring InsetCommandParams::getFirstNonOptParam() const
 }
 
 
+bool InsetCommandParams::hasParam(std::string const & name) const
+{
+	return info_.hasParam(name);
+}
+
+
+docstring const & InsetCommandParams::getParamOr(std::string const & name, docstring const & defaultValue) const
+{
+	if (hasParam(name))
+		return (*this)[name];
+	return defaultValue;
+}
+
+
 docstring const & InsetCommandParams::operator[](string const & name) const
 {
 	static const docstring dummy;
-	LASSERT(info_.hasParam(name), return dummy);
+	LASSERT(hasParam(name), return dummy);
 	ParamMap::const_iterator data = params_.find(name);
 	if (data == params_.end() || data->second.empty())
 		return dummy;
@@ -615,7 +655,7 @@ docstring const & InsetCommandParams::operator[](string const & name) const
 
 docstring & InsetCommandParams::operator[](string const & name)
 {
-	LATTEST(info_.hasParam(name));
+	LATTEST(hasParam(name));
 	// this will add the name in release mode
 	ParamInfo::ParamData const & param = info_[name];
 	if (param.ignore())
@@ -627,6 +667,7 @@ docstring & InsetCommandParams::operator[](string const & name)
 void InsetCommandParams::clear()
 {
 	params_.clear();
+	preview(false);
 }
 
 
