@@ -25,8 +25,7 @@ namespace lyx {
 
 class Buffer;
 class MathData;
-class MathMacroTemplate;
-class Paragraph;
+class InsetMathMacroTemplate;
 class latexkeys;
 
 enum MacroType {
@@ -39,11 +38,11 @@ enum MacroType {
 class MacroData {
 public:
 	/// Constructor to make STL containers happy
-	MacroData(Buffer * buf = 0);
+	explicit MacroData(Buffer const * buf = 0);
 	/// Create lazy MacroData which only queries the macro template when needed
-	MacroData(Buffer * buf, DocIterator const & pos);
+	MacroData(Buffer const * buf, DocIterator const & pos);
 	/// Create non-lazy MacroData which directly queries the macro template
-	MacroData(Buffer * buf, MathMacroTemplate const & macro);
+	MacroData(Buffer const * buf, InsetMathMacroTemplate const & macro);
 
 	///
 	docstring const & definition() const { updateData(); return definition_; }
@@ -60,7 +59,7 @@ public:
 	///
 	std::vector<docstring> const & defaults() const;
 	///
-	std::string const requires() const;
+	std::string const required() const;
 	///
 	bool hidden() const;
 	///
@@ -68,9 +67,11 @@ public:
 	///
 	char const * MathMLtype() const;
 	///
+	latexkeys const * symbol() const { return sym_; }
+	///
 	void setSymbol(latexkeys const * sym) { sym_ = sym; }
 	///
-	DocIterator const & pos() { return pos_; }
+	DocIterator const & pos() const { return pos_; }
 
 	/// lock while being drawn to avoid recursions
 	int lock() const { return ++lockCount_; }
@@ -104,7 +105,7 @@ public:
 
 private:
 	///
-	void queryData(MathMacroTemplate const & macro) const;
+	void queryData(InsetMathMacroTemplate const & macro) const;
 	///
 	void updateData() const;
 	///
@@ -119,25 +120,25 @@ private:
 	/// macros.
 	mutable DocIterator pos_;
 	///
-	mutable bool queried_;
+	mutable bool queried_ = false;
 	///
 	mutable docstring definition_;
 	///
-	mutable size_t numargs_;
+	mutable size_t numargs_ = 0;
 	///
 	mutable docstring display_;
 	///
-	latexkeys const * sym_;
+	latexkeys const * sym_ = nullptr;
 	///
-	mutable size_t optionals_;
+	mutable size_t optionals_ = 0;
 	///
 	mutable std::vector<docstring> defaults_;
 	///
-	mutable int lockCount_;
+	mutable int lockCount_ = 0;
 	///
-	mutable bool redefinition_;
+	mutable bool redefinition_ = false;
 	///
-	mutable MacroType type_;
+	mutable MacroType type_ = MacroTypeNewcommand;
 };
 
 
